package com.onior.pdf;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.pdfbox.cos.COSArray;
import org.apache.pdfbox.cos.COSDictionary;
import org.apache.pdfbox.cos.COSName;
import org.apache.pdfbox.cos.COSNumber;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.interactive.action.PDAction;
import org.apache.pdfbox.pdmodel.interactive.action.PDActionGoTo;
import org.apache.pdfbox.pdmodel.interactive.documentnavigation.destination.PDDestination;
import org.apache.pdfbox.pdmodel.interactive.documentnavigation.destination.PDPageDestination;
import org.apache.pdfbox.pdmodel.interactive.documentnavigation.outline.PDDocumentOutline;
import org.apache.pdfbox.pdmodel.interactive.documentnavigation.outline.PDOutlineItem;

public class PageDestinationOffset {

	public static void main( String[] args ) throws Exception {
		String filepath = args[0];
		System.out.println( filepath );
		InputStream source = new FileInputStream( filepath + "example.pdf" );
		OutputStream output = new FileOutputStream( filepath + "output.pdf" );
		PDDocument document = null;
		try {
			document = PDDocument.load( source );
			PDDocumentOutline outline = document.getDocumentCatalog().getDocumentOutline();
			if ( outline != null ) {
				int pagenumber = getPageIndexForFirstBookmark( outline.getFirstChild() );
				System.out.println( "Bookmark found for page " + pagenumber );

				PDOutlineItem bookmark = new PDOutlineItem();
				bookmark.setTitle( "Bookmark test" );
				COSDictionary actionDict = new COSDictionary();
				actionDict.setName( COSName.S, "GoTo" );
				COSArray destinationArray = new COSArray();
				destinationArray.add( 0, COSNumber.get( String.valueOf( pagenumber ) ) );
				destinationArray.add( 1, COSName.getPDFName( "Fit" ) );
				actionDict.setItem( "D", destinationArray );
				bookmark.setAction( new PDActionGoTo( actionDict ) );
				System.out.println( "Created new bookmark on same page" );
				outline.addLast( bookmark );

				document.save( output );
			} else {
				System.out.println( "No bookmarks found" );
			}
		} finally {
			if ( output != null ) {
				output.close();
			}
			if ( source != null ) {
				source.close();
			}
			if ( document != null ) {
				document.close();
			}
		}
	}

	static int getPageIndexForFirstBookmark( PDOutlineItem item ) throws IOException {
		if ( item != null ) {
			PDDestination destination = item.getDestination();
			PDAction action = item.getAction();
			if ( action instanceof PDActionGoTo && destination == null ) {
				PDActionGoTo gotoAction = (PDActionGoTo) action;
				destination = gotoAction.getDestination();
			}
			if ( destination != null ) {
				if ( destination instanceof PDPageDestination ) {
					PDPageDestination pageDest = (PDPageDestination) destination;
					return pageDest.findPageNumber();
				}
			} else {
				int retval = getPageIndexForFirstBookmark( item.getFirstChild() );
				if ( retval > 0 ) {
					return retval;
				}
			}
			return getPageIndexForFirstBookmark( item.getNextSibling() );
		}
		return -1;
	}
}
